<?php
declare( strict_types=1 );
namespace WebFacing\cPanel\Email;
use WebFacing\cPanel\UAPI;

/**
 * Exit if accessed directly
 */
\class_exists( 'WP' ) || exit;

abstract class Main {

	public    const pf = 'wf_cpanel_email_';

	private   const proisp_packages = [
		'prostart'      => 'Pro Start',
		'promedium'     => 'Pro Medium',
		'propremium'    => 'Pro Premium',
		'enterprise10'  => 'Enterprise 10',
		'enterprise30'  => 'Enterprise 30',
		'enterprise60'  => 'Enterprise 60',
		'enterprise100' => 'Enterprise 100',
	];

	public    static \stdClass  $plugin;

	public    static string     $pf;

	public    static bool       $is_debug = false;

	private   static ?string    $dev_email = null;

	private   static ?\WP_User  $dev_user = null;

	private   static bool       $did_init = false;

	protected static \stdClass  $page;

	protected static \WP_Screen $screen;

	protected static bool       $is_cpanel = false;

	public    static bool       $has_exec;

	public    static bool       $has_http;

	public    static bool       $use_exec = true;

	public    static bool       $is_subadmin = false;

	public    static string     $site_domain = '';

	public    static string     $main_domain = '';

	public    static int        $main_parts = 0;

	public    static bool       $domain_only = false;

	public    static bool       $remote_cpanel = false;

	public    static string     $host_name = 'localhost';

	public    static bool       $is_proisp = false;

	public    static float      $cpanel_version;

	public    static string     $cpanel_user;

	public    static array      $transients = [];

	public    static string     $active_key = self::pf . 'token.active.';

	public    static int        $transient_time = 20 * \MINUTE_IN_SECONDS;

	public    static string     $plan;

	public    static string     $package;

	public    static int        $bytes_used;

	public    static int        $byte_limit;

	protected static function init(): void {

		if ( ! self::$did_init ) {

			self::$pf          = \trim( \str_replace( '_', '-', self::pf ) );

			self::$plugin      = (object) \get_plugin_data( PLUGIN_FILE );

			self::$is_debug    = \defined( 'WF_DEBUG' ) && \WF_DEBUG;

			self::$cpanel_user = \defined( 'WF_CPANEL_USER' ) ? \WF_CPANEL_USER : \explode( \DIRECTORY_SEPARATOR, \ABSPATH )[2];

			self::$active_key = self::pf . 'token.active.' . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' );

			self::$site_domain = \wp_parse_url( \site_url(), \PHP_URL_HOST );

			$saddr = $_SERVER['SERVER_ADDR'] ?? '';

			self::$remote_cpanel = \defined( 'WF_CPANEL_HOST' ) && ! empty( \WF_CPANEL_HOST ) && ! empty( self::$cpanel_user );

			if ( self::$remote_cpanel ) {
				self::$host_name = \WF_CPANEL_HOST;
				self::$has_exec = false;
			} else {

				if ( empty ( $saddr ) ) {
					self::$host_name = self::$site_domain;
				} else {
					self::$host_name  = \gethostbyaddr( $saddr ) ?: '';
				}
				self::$has_exec = UAPI::set_shell();
			}

			self::$has_http = self::set_token();

			self::$use_exec = ! self::$remote_cpanel && self::$has_exec && get_option( self::$active_key ) === UAPI::exec;

			self::$is_cpanel   = ( self::$has_http || self::$has_exec ) && UAPI::has_features();

			if ( self::$is_cpanel ) {
				UAPI::$cpanel_user = UAPI::user() ?? self::$cpanel_user;
				self::$active_key = self::pf . 'token.active.' . UAPI::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' );
			}

			self::$did_init = true;
		}
	}

	public    static function load(): void {

		if ( \apply_filters( self::pf . 'frontend', false ) ) {
			self::init();
		}
		ShortCode::load();
	}

	public    static function admin(): void {

		self::$page        = new \stdClass;

		self::$dev_email = \defined( 'WF_DEV_EMAIL' ) &&
			\is_string( \WF_DEV_EMAIL ) &&
			\str_contains( \WF_DEV_EMAIL, '@' ) &&
			\strlen( \WF_DEV_EMAIL ) > 5 ?
			\sanitize_email( \WF_DEV_EMAIL ) :
			null;

		self::init();

		if ( self::$is_cpanel ) {
			if ( ! \wp_doing_ajax() ) {
				UAPI::set_locale();
			}

			self::$main_domain = UAPI::main_domain();

			self::$main_parts  = \count( \explode( '.', self::$main_domain ) );

			self::$plan        = UAPI::user( 'plan' );

			$host_ids = \explode( '.', self::$host_name );
			$num_f    = \count( $host_ids );
			$host_id  = $num_f > 1 && \array_key_exists( $num_f - 1, $host_ids ) ? $host_ids[ $num_f - 2] . '.' . $host_ids[ $num_f - 1] : null;
			self::$is_proisp     = \in_array( $host_id, [ 'proisp.no', 'proisp.eu' ], true );

			self::$package     = self::$is_proisp ? ( self::proisp_packages[ self::$plan ] ?? '' ) : self::$plan;
		}

		\add_action( 'init', static function(): void {
			/**
			 * Set this developer user
			 */
			$parts = \explode( '/', \rtrim( self::$plugin->AuthorURI, ' /' ) );
			$id = self::$dev_email ?? \end( $parts );
			self::$dev_user = \get_user_by( self::$dev_email ? 'email' : 'login', $id ) ?: null;

			self::$is_subadmin = \is_multisite() && ! \is_super_admin();

			$site_domain_only_defined =
				\defined( 'WF_CPANEL_EMAIL_SITE_DOMAIN_ONLY' ) || \defined( 'WF_CPANEL_EMAIL_DOMAIN_ONLY' )
			;
			$site_domain_only =
				( \defined( 'WF_CPANEL_EMAIL_SITE_DOMAIN_ONLY' ) && \WF_CPANEL_EMAIL_SITE_DOMAIN_ONLY ) ||
				( \defined( 'WF_CPANEL_EMAIL_DOMAIN_ONLY'      ) && \WF_CPANEL_EMAIL_DOMAIN_ONLY      )
			;
			self::$domain_only = $site_domain_only_defined ?
				$site_domain_only :
				self::$is_subadmin || $site_domain_only
			;

			self::$domain_only = \apply_filters( self::pf . 'site_domain_only', self::$domain_only );
		}, 11 );

		\add_action( 'current_screen', static function(): void {
			self::$screen = \get_current_screen();
		}, 0 );

		\add_action( 'admin_notices', static function(): void {
			$user_email  = \wp_get_current_user()->user_email;
			$user_domain = \explode( '@', $user_email )[1] ?? '@';
			$cap = \apply_filters( self::pf . 'capability', 'manage_options', self::$screen->id );

			if ( \current_user_can( $cap ) ) {
			} elseif ( \str_ends_with( self::$site_domain, $user_domain ) ) {
				$cap = 'cpanel';
			}

			if ( self::$is_cpanel ) {
			} elseif ( \current_user_can( $cap ) ) { ?>
				<div class="notice notice-error">
					<p>
<?php
				_ex( 'cPanel® Email features not available!', 'Notice Error Message' );

				if ( \current_user_can( 'deactivate_plugins' ) ) { ?>
					</p><p>
<?php
					if ( self::$has_http || self::$has_exec ) {
						\printf(
							/* translators: 1: This Plugin Name */
							_x( 'cPanel® not detected, or no email features available! You should just deactivate the "%1$s" plugin because it can\'t work on this host.', 'Notice Error Message' ),
							self::$plugin->Name
						);
					} else {
						\printf(
							/* translators: 1: shell_exec, 2: constant, 3: wp-config.php, 4: Documentation link */
							_x( 'cPanel® not yet detected, since the %1$s function is disabled and no valid API token configured. If <strong>on cPanel®</strong>, please log in on it, create a temporary token and add it in a constant %2$s in your %3$s file. See %4$s on how to create one. Later, tokens can be managed from your Admin area.', 'Notice Error Message' ),
							'<code>' . UAPI::exec . '</code>',
							'<code>WF_CPANEL_API_TOKEN</code>',
							'<code>wp-config.php</code>',
							'<a href="https://docs.cpanel.net/cpanel/security/manage-api-tokens-in-cpanel/" target="_blank" rel="external noreferrer noopener">Manage API Tokens in cPanel®</a>'
						);
					}
				} else {
					_ex( 'Please contact your administrator.', 'Notice Error Message' ); ?>
<?php
				} ?>
					</p>
				</div>
<?php
			}

			if ( self::$is_debug ) {

				if ( self::$screen->id !== 'site-health' && self::$dev_user instanceof \WP_User && self::$dev_user->ID === \get_current_user_id() ) { ?>
					<div class="notice notice-info is-dismissible">
						<p>
							<?php echo self::$plugin->Name, ' ', self::$plugin->Version , ' ', self::$cpanel_user; ?><br/>
							<?='API: '.UAPI::$response_message?>, <?=UAPI::exec,' ',self::$has_exec?'':'not ','allowed'?><br/>
							<?php //var_dump( \shell_exec( "uapi Backup restore_files backup='/home/seniodmz/public_html/wp-content/cpanel/backup-11.8.2022_15-51-17_seniodmz-MtpXnkUw.tar.gz'" ) ); ?><!--br/-->
						</p>
					</div>
<?php 				error_log( [ 'version' => self::$plugin->Version, 'dev_email' => self::$dev_user->user_email ] );
				}
			}
		} );

		if ( self::$is_cpanel ) {

			self::$bytes_used = (int) UAPI::quotas()->bytes_used;

			self::$byte_limit = (int) UAPI::quotas()->byte_limit;

			AccountsPage::admin();
			NewEmail::admin();
			BoxesPage::admin();
			BackupsPage::admin();
			ContactsPage::admin();
			TokensPage::admin();
			SiteHealth::admin();

			\add_filter( 'dashboard_glance_items', static function( ?array $elements ): array {

				if ( \current_user_can( 'publish_pages' ) ) {
					$elements = $elements ?? [];
					$href = \add_query_arg( [ 'page' => self::$pf . 'accounts' ], \admin_url( 'admin.php' ) );
					// Protect against duplicate from my other plugin
					$dup = \preg_grep( '/ class="email-count"/', $elements );

					if ( \count( $dup ) ) {
						unset( $elements[ \array_keys( $dup)[0] ] );
					}

//					$email_accounts = UAPI::email_accounts();
//					$num_accounts = 1 + ( \is_array( $email_accounts ) ? \count( $email_accounts ) : 0 );
					$num_accounts = UAPI::count_emails();
					$elements[ self::$pf ] = '<a href="' . $href . '" class="cpanel-email-count">' . $num_accounts . ' ' . _nx( 'cPanel® email account used', 'cPanel® email accounts used', $num_accounts, 'Right Now' ) . '</a>';

					// Protect against duplicate from my other plugin
					$dup = \preg_grep( '/" class="disk-count /', $elements );

					if ( \count( $dup ) ) {
						unset( $elements[ \array_keys( $dup)[0] ] );
					}
					$elements[] = '<a href="' . $href . '" class="cpanel-disk-used" title="' . _x( 'cPanel® account storage space limit:', 'Right Now' ) . ' ' . \size_format( self::$byte_limit ) . '">' . \size_format( self::$bytes_used, 1 ) . ' ' . _x( 'cPanel® space used', 'Right Now' ) . '</a>';
				}
				return $elements;
			}, 112 );

			\add_action( 'rightnow_end', static function(): void {

				if ( \current_user_can( 'activate_plugins' ) && ! \class_exists( 'WebFacing\cPanel\Main' ) ) {
					echo \PHP_EOL, \wpautop( \sprintf(
						__( 'Hosted on a %1$s plan with %2$s storage space limit.' ),
						self::$package,
						\size_format( self::$byte_limit )
					) );
				}
			} );

			/*
			 * Custom Icon for Disk space in "At a Glance" widget
			 */
			\add_action( 'admin_head', static function(): void {

				if ( self::$screen->id === 'dashboard' ) { ?>
					<style>
						#dashboard_right_now li a.cpanel-email-count:before {
							content: '\f465';
							margin-left: -1px;
						}
						#dashboard_right_now li a.cpanel-disk-used:before {
							content: '\f17e';
							margin-left: -1px;
						}
					</style>
<?php
				}
			} );

	//		Delete our expired transients
			\add_action( 'admin_footer', static function(): void {
				global $wpdb;

				if ( \idate( 's' ) === 0 ) {
					$pf        = self::$pf;
					$threshold = \time() - \MINUTE_IN_SECONDS;
					$sql       = "
						DELETE FROM `T1`, `T2`
						USING `{$wpdb->options}` AS `T1`
						JOIN `{$wpdb->options}` AS `T2` ON `T2`.`option_name` = REPLACE( `T1`.`option_name`, '_timeout', '' )
						WHERE ( `T1`.`option_name` LIKE '_transient\_timeout\_{$pf}%' OR `T1`.`option_name` LIKE '_site\_transient\_timeout\_{$pf}%' )
						AND `T1`.`option_value` < '{$threshold}'
					;";
					$rows = \ceil( $wpdb->query( $sql ) / 2 );
					\do_action( 'qm/info', 'Deleted {rows} transients.', [ 'rows' => $rows ] );

					$transients = (array) \get_transient( self::$pf . 'transients' );
					$transients = \array_unique( \array_merge( self::$transients, \is_array( $transients ) ? $transients : [] ) );
					\set_transient( self::$pf . 'transients', $transients );
					\do_action( 'qm/debug', $transients );
				}
			} );
		}
	}

	public    static function set_token( string $raw_token = '' ): bool {

		if ( empty( $raw_token ) ) {
//			$active_key = self::pf . 'token.active.' . self::$cpanel_user;
			$active_key = self::$active_key;
			$token = get_option( $active_key );

			if ( $token ) {
				$aknown_key = self::pf . 'token.' . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name  : '' ) . '.' . $token;
				$raw_token  = get_option( $aknown_key );

				if ( $raw_token ) {
					$raw_token = UAPI::validate_token( $raw_token ) ? $raw_token : '';
				}

				if ( empty( $raw_token ) ) {

					if ( self::$has_exec ) {
						update_option( $active_key, UAPI::exec );
					} else  {
						delete_option( $active_key );
					}
				}
			}
		}

		if ( empty( $raw_token ) ) {
			$raw_token = UAPI::get_defined_api_token();
			$raw_token = UAPI::validate_token( $raw_token ) ? $raw_token : '';
		}

		if ( empty( $raw_token ) ) {
			return false;
		}
		UAPI::$api_token = self::$cpanel_user . ':' . $raw_token;
		return true;
	}

	public    static function delete_transients(): void {
		$x_transient_names = [
			self::$pf . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' ) . 'Email-list_forwarders',
			self::$pf . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' ) . 'Email-list_forwarders-',
			self::$pf . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' ) . 'Email-list_pops',
			self::$pf . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' ) . 'Email-list_pops-',
			self::$pf . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' ) . 'Email-list_pops_with_disk',
			self::$pf . self::$cpanel_user . ( self::$remote_cpanel ? '@' . self::$host_name : '' ) . 'Email-list_pops_with_disk-',
		];
		$transients = (array) \get_transient( self::$pf . 'transients' );
		$transients = \array_unique( \array_merge( self::$transients, $transients, $x_transient_names ) );
		\do_action( 'qm/info', $transients );

		foreach ( (array) $transients as $transient ) {
			\delete_transient( $transient );
		}
		\delete_transient( self::$pf . 'transients' );
	}

	public    static function email_mx_self( string $domain = '' ): bool {
		$domain = $domain ?: self::$site_domain;
		$hostparts = \explode( '.', self::$host_name );
		$hostpartscount = \count( $hostparts );

		if ( $hostpartscount < 2 ) {
			error_log ( $hostparts );
		}
		$hostends = ( $hostpartscount > 1 ? $hostparts[ $hostpartscount - 2] . '.' : '' ) . $hostparts[ $hostpartscount - 1];
		$hosts = [];
		$result = \getmxrr( $domain, $hosts );
		return $result && \array_key_exists( 0, $hosts ) && ( \str_ends_with( $hosts[0], $domain ) || \str_ends_with( $hosts[0], $hostends ) );
	}

	public    static function email_to_utf8( string $email ): string {

		if ( \is_email( $email ) && \function_exists( 'idn_to_utf8' ) ) {
			$parts = \explode( '@', $email, 2 );
			return $parts[0] . '@' . \idn_to_utf8( $parts[1] );
		} else {
			return $email;
		}
	}

	public    static function send_password( string $to_email, string $domain, string $password ): bool {
		return false;
	}
}
