<?php
declare( strict_types=1 );
namespace WebFacing\cPanel\Email;
use WebFacing\cPanel\UAPI;

/**
 * Exit if accessed directly
 */
\class_exists( __NAMESPACE__ . '\Main' ) || exit;

abstract class NewEmail extends Main {

	public    static int     $tz_offset = 0;

	protected static bool    $batch_mode = false;

	private   static array   $batch_entries = [];

	public    static function admin(): void {

		self::$page->NewEmail = self::$pf . 'new-email';

		self::$tz_offset = \wp_timezone()->getOffset( new \DateTime );

		\add_action( 'admin_init', static function(): void {
			self::save();
		} );

		\add_action( 'current_screen', static function(): void {

			\add_action( 'load-'. self::$screen->id, static function(): void {

				if ( $_GET['page'] === self::$page->NewEmail ) {

					$options = [
//						'batch-entry'       => [                                 _x( 'Batch Entry',   'Form Heading' )  , false ],
						'new-forward'       => [ \sprintf( __(    'From %1$s' ), _x( 'New Forwarder', 'Form Heading' ) ),  true ],
						'default-forward'   => [ \sprintf( __( 'Default %1$s' ), _x( 'New Forwarder', 'Form Heading' ) ), false ],
						'new-failure'       => [ \sprintf( __(    'From %1$s' ), _x( 'New Failure',   'Form Heading' ) ), false ],
						'default-failure'   => [ \sprintf( __( 'Default %1$s' ), _x( 'New Failure',   'Form Heading' ) ), false ],
						'new-blackhole'     => [ \sprintf( __(    'From %1$s' ), _x( 'New Blackhole', 'Form Heading' ) ), false ],
						'default-blackhole' => [ \sprintf( __( 'Default %1$s' ), _x( 'New Blackhole', 'Form Heading' ) ), false ],
						'new-account'       => [                     _x( 'New Account &amp; Mailbox', 'Form Heading' )  ,  true ],
						'new-responder'     => [                     _x( 'New Autoresponder',         'Form Heading' )  , false ],
					];

					foreach ( $options as $option => $lbl_def ) {

						$opt     = \str_replace(
							[ 'default', 'forward', 'failure', 'blackhole', 'account', 'responder' ],
							[ 'def',     'forw',    'fail',    'black',     'acc',     'resp'      ],
							$option,
						);
						$option = 'wc-show-' . $option;
						$default = \apply_filters( self::$page->NewEmail .'_user-option', $lbl_def[1], $option, \get_current_user() );
						self::$screen->add_option( $opt,  [ 'option' => $option, 'default' => $default ] );
					}
					self::save_options();

					foreach ( $options as $option => $lbl_def ) {

						$opt     = \str_replace(
							[ 'default', 'forward', 'failure', 'blackhole', 'account', 'responder' ],
							[ 'def',     'forw',    'fail',    'black',     'acc',     'resp'      ],
							$option,
						);
						$option = 'wc-show-' . $option;
						$label   = $lbl_def[0];
						$default = \apply_filters( self::$page->NewEmail .'_user-option', $lbl_def[1], \get_current_user() );
						$value   = \get_user_option( $option );
						$value   = $value === false ? $default : $value;
						self::$screen->add_option( $opt,  [ 'label' => $label, 'option' => $option, 'value' => $value, 'default' => $default ] );
					}
					$options = self::$screen->get_options();
					self::$batch_mode = false;//\boolval( $options['batch-entry']['value'] );
//					self::save();
				}
			} );

			\add_filter( 'screen_settings', static function( string $html, \WP_Screen $screen ): string {

				if ( ( $_GET['page'] ?? '' ) === self::$page->NewEmail ) {
					$html .= \PHP_EOL . ' <fieldset class="' . self::$pf . '">';
					$html .= \PHP_EOL . '  <legend>' . _x( 'Show Forms:', 'Screen Options Legend' ) . '</legend>';
					$html .= \PHP_EOL . '  <ul style="margin: 0;">';

					foreach ( $screen->get_options() as $option ) {
						$html .= \PHP_EOL . '   <li style="float: left; padding-right: 1ch;"><label style=" line-height: 1;"><input type="checkbox" name="' . $option['option'] . '"' . \checked( $option['value'], true, false ) . 'value="' . true . '"/> ' . $option['label'] . '</label></li>';
					}
					$html .= \PHP_EOL . '  </ul>';
					$html .= \PHP_EOL . '  </fieldset>';
					$html .= \PHP_EOL . '<button type="submit" name="action" value="user-options" class="button button-secondary">' . _x( 'Save Settings', 'Screen Options Button Text' ) . '</button>';
				}
				return $html;
			}, 10, 2 );
		} );

		\add_action( 'admin_bar_menu', static function( \WP_Admin_Bar $bar ): void {
			$bar->add_node( [ 'parent' => 'new-content', 'id' => 'cpanel', 'title' => _x( 'Email address', 'Toolbar label' ), 'href' => \add_query_arg( [ 'page' => self::$pf . 'new-email' ], \admin_url( 'admin.php' ) ) ] );
		}, 121 );

		\add_action( 'admin_menu', static function(): void {

			\add_submenu_page(
				self::$page->AccountsPage,
				_x( 'cPanel® Add New Email Address', 'Page Title' ),
				_x( 'Add New Email', 'Submenu Label' ),
				\apply_filters( self::pf . 'capability', 'manage_options' ),
				self::$page->NewEmail,
				static function(): void {
					$mail_domains = UAPI::mail_domains();

					if ( self::$domain_only ) {
						$mail_domains = \array_filter( $mail_domains, static function( string $mail_domain ): bool {
							return $mail_domain === self::$site_domain;
						} );
					}
					$readonly = \count( $mail_domains ) === 1 ? ' readonly="readonly"' : '';

					$dest_emails = \wp_list_pluck( \array_filter( (array) UAPI::email_forwarders(), static function( \stdClass $account ): bool {
						return (bool) \is_email( $account->dest );
					} ), 'dest' );
					$accounts = (array) UAPI::email_accounts();
					$used_emails = \array_merge( \wp_list_pluck( $accounts, 'email' ), [ \sanitize_email( UAPI::main_email_account()->email ) ] );

					if ( self::$domain_only ) {
						$accounts = \array_filter( $accounts, static function( \stdClass $account ): bool {
							return \str_ends_with( $account->email, '@' . self::$site_domain );
						} );
						$to_emails = \array_unique( \array_merge( \wp_list_pluck( $accounts, 'email' ), [ \wp_get_current_user()->user_email ], [ \get_bloginfo( 'admin_email' ) ] ) );
					} else {
						$to_emails = \array_unique( \array_merge( \wp_list_pluck( $accounts, 'email' ), $dest_emails, [ \wp_get_current_user()->user_email ], [ \get_bloginfo( 'admin_email' ) ], [ \sanitize_email( UAPI::main_email_account()->email ) ] ) );
					}

					$action  = \sanitize_key( $_GET['action'] ?? '' );
					$type    = \sanitize_key( $_GET['type'  ] ?? '' );
					$domain  = \sanitize_text_field( $_GET['domain'] ?? '' );
					$email   = \sanitize_email(      $_GET['email' ] ?? '' );
					$pattern = '^[A-Za-z0-9_!#$%&\'*+/=?`{|}~^-]+(?:\.[A-Za-z0-9_!#$%&\'*+/=?`{|}~^-]+↵)*$';
//					$pattern = '[a-zA-Z0-9](_(?!(\.|\-\_))|\.(?!(\_|\.))|[a-zA-Z0-9]){0,96}[a-zA-Z0-9]';
//					$pattern = '^_`{|}~-]+(?:\.[a-z0-9!#$%&\'*+/=?^_`{|}~-]+)$';
					$no_user = 'No Such User Here';
					$in_user = __( 'No Such User Here', 'Fail Message' );
					$edit    = false;

					if ( $action === 'row-edit' ) {
						$edit = $type;
					}

					if ( $edit === 'responder' ) {
						$responder = UAPI::email_responder( $email );
						$responder->user       = \explode( '@', $email )[0];
						$responder->start_date = \date_i18n( 'Y-m-d', $responder->start );
						$responder->start_time = \date_i18n( 'H:i',   $responder->start );
						$responder->stop_date  = \date_i18n( 'Y-m-d', $responder->stop  );
						$responder->stop_time  = \date_i18n( 'H:i',   $responder->stop  );
						$to_emails = \array_unique( \array_merge( $to_emails, [ \sanitize_email( $responder->from ) ] ) );
					} else {
						$responder = new \stdClass;
						$responder->from       = \wp_get_current_user()->display_name;
						$responder->subject    = 'Re: %subject%';
						$responder->body       = \str_replace( '%div%', '--  ', \sprintf(
							_x( 'Hello %%from%% %%email%%,


%%div%%
Regards,
%%responder%%
%1$s',
								'Email Body Suggestion, %responder% will be expanded to Responder Name, %1$s = Site Name'
							),
							\get_bloginfo()
						) );
						$responder->user       = '';
						$responder->start_date = \date_i18n( 'Y-m-d' );
						$responder->start_time = \date_i18n( 'H:i' );
						$responder->stop_date  = \date_i18n( 'Y-m-d', \strtotime( '+1 weeks', \current_time( 'timestamp' ) ) );
						$responder->stop_time  = \date_i18n( 'H:i' );
						$responder->interval   = 24;
						$responder->is_html    = 0;
					}
					$options = self::$screen->get_options();
//					self::$batch_mode = \boolval( $options['batch-entry']['value'] ?? '' );

					if ( self::$batch_mode ) {
						$options['def-forw' ]['value'] = false;
						$options['new-fail' ]['value'] = false;
						$options['def-fail' ]['value'] = false;
						$options['new-black']['value'] = false;
						$options['def-black']['value'] = false;
						$options['new-resp' ]['value'] = false;
						$one = false;

						foreach ( [ 'new-forw', 'new-acc' ] as $class ) {
							$one = $one || $options[ $class ]['value'];
						}

						if ( ! $one ) {
							$options['new-forw']['value'] = true;
						}
					} ?>
		<div class="wrap">
			<style scoped="scoped">
<?php
					foreach ( $options as $class => $option ) {

						if ( ! $option['value'] ) {
							echo '.', $class, ' { display: none; }', \PHP_EOL;
						} else {
							echo '.', $class, ' { display: block !important; }', \PHP_EOL;
						}
					} ?>
				.batch-entry { display: none; }
				.form-table {
					width: 120ch;
				}
				.form-table th, .form-table td {
					padding: 0 1ch;
				}
				.form-table tbody th:nth-of-type(1), .form-table tbody td:nth-of-type(1) {
					width: 15%;
					white-space: nowrap;
					padding-right: 0;
					text-align: right;
				}
				.form-table tbody td input, .form-table tbody td select {
					width: calc( 100% - 2ch);
				}
				.form-table tbody td input:focus:invalid {
					background-color: LightYellow;
					border-color: Red;
				}
				.form-table thead th:nth-of-type(2), .form-table tbody td:nth-of-type(2) {
					width: 25%;
					padding-left: 0;
				}
				.form-table th:nth-of-type(3), .form-table td:nth-of-type(3) {
					width: 30%;
				}
				.form-table th:last-of-type, .form-table td:last-of-type {
					text-align: left;
				}
			</style>
			<h1 class="wp-heading-inline"><?php echo \get_admin_page_title(); ?></h1>
			<p><?=__('NEW: Open Screen Options drawer above, tick boxes and save, to see hidden forms on this screen.')?></p>
			<hr class="wp-header-end"/>
			<p><br /></p>
<?php			if ( ! $edit ) { ?>
			<h2 class="batch-entry">֎ <?php _ex( 'Batch Entry', 'Form Heading' ); ?></h2>
			<form id="batch-entry" class="batch-entry" name="batch-entry" method="post" action="">
				<input type="hidden" name="action" value="batch-entry">
				<textarea name="entries" cols="100"><?php foreach ( self::$batch_entries as $e ) { echo $e, \PHP_EOL; }; ?></textarea>
			</form>
			<h2 class="new-forw def-forw">↪ <?php _ex( 'New Forwarder', 'Form Heading' ); ?></h2>
			<form id="new-forward" class="new-forw" name="new-forward" method="post" action="">
				<input type="hidden" name="action" value="new-forward">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="forward-user"><?php _ex( 'From Email', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
							<th scope="col">
								<label for="forward-to"><?php _ex( 'To Emails <small>(comma separated)</small>', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="forward-user" name="user" class="user" autocomplete="on" required="required" pattern="<?php echo $pattern; ?>"/>@
							</td>
							<td>
								<select id="forward-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
							</td>
							<td>
								<input type="text" id="forward-to" name="forward" autocomplete="on" required="required"/>
							</td>
							<td>
								<button type="submit" class="button-primary"><?php _ex( 'Add Forwarder', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			}
				$def_msg = $domain ? \trim( \str_replace( ':fail:','', UAPI::default_address( $domain ) ), ' "' ) : '';
				if ( $edit !== 'responder' ) { ?>
			<form id="default-forward" class="def-forw" name="default-forward" method="post" action="">
				<input type="hidden" name="action" value="default-forward">
				<table class="form-table">
					<thead>
						<tr>
							<th></th>
							<th scope="col">
								<label for="default-forward-domain"><?php _ex( 'From Domain', 'Form Field Label' ); ?></label>
							</th>
							<th scope="col">
								<label for="forward"><?php _ex( 'To Email', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td class="default">*@</td>
							<td>
								<select id="default-forward-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td>
								<input type="email" id="forward" name="forward" autocomplete="on" required="required" value="<?php echo \is_email( $def_msg ) ? $def_msg : ''; ?>"/>
							</td>
							<td>
								<button type="submit" class="button-primary"><?php _ex( 'Change Default to Forwarder', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			}

				if ( ! $edit ) { ?>
			<p class="new-forw def-forw"><?php _e( '&mdash; OR &mdash;' ); ?></p>
			<h2 class="new-fail def-fail">🚫 <?php _ex( 'New Failure', 'Form Heading' ); ?></h2>
			<form id="new-failure" class="new-fail" method="post" action="">
				<input type="hidden" name="action" value="new-failure">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="fail-user"><?php _x( 'For Email', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
							<th scope="col">
								<label for="fail-msg"><?php _ex( 'Failure Message', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="fail-user" name="user" autocomplete="on" required="required" pattern="<?php echo $pattern; ?>"/>@
							</td>
							<td>
								<select id="fail-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td>
								<input type="text" id="fail-msg" name="msg" autocomplete="off" placeholder="<?php echo $in_user; ?>" list="msg-list"/>
							</td>
							<td>
								<button type="submit" class="button-primary" name="type" value="email"><?php _ex( 'Add Failure', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			}
				if ( $edit !== 'responder' ) { ?>
			<form id="default-fail" class="def-fail" name="default-fail" method="post" action="">
				<input type="hidden" name="action" value="default-fail">
				<table class="form-table">
					<thead>
						<tr>
							<th></th>
							<th scope="col">
								<label for="default-fail-domain"><?php _ex( 'For Domain', 'Form Field Label' ); ?></label>
							</th>
							<th scope="col">
								<label for="default-fail-msg"><?php _ex( 'Failure Message', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>*@</td>
							<td>
								<select id="default-fail-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td>
								<input type="text" id="default-fail-msg" name="msg" autocomplete="off" placeholder="No Such User Here" value="<?php echo \is_email( $def_msg ) ? '' : $def_msg; ?>" list="msg-list"/>
								<datalist id="msg-list">
									<option><?php echo $in_user === $no_user ? $no_user : $in_user . '</option><option>' . $no_user; ?></option>
								</datalist>
							</td>
							<td>
								<button type="submit" class="button-primary"><?php echo $edit ? _x( 'Save Default Failure', 'Button' ) : _x( 'Change Default to Failure', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			}

				if ( ! $edit ) { ?>
			<p class="new-fail def-fail"><?php _e( '&mdash; OR &mdash;' ); ?></p>
			<h2 class="new-black def-black">⚫ <?php _ex( 'New Blackhole', 'Form Heading' ); ?></h2>
			<form id="new-black" class="new-black" method="post" action="">
				<input type="hidden" name="action" value="new-black">
				<table class="form-table">
					<thead>
						<tr>
							<th colspan="4" scope="col">
								<label for="black-user"><?php _ex( 'For Email', 'Form Field Label' ); ?></label>
							</th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<datalist id="black-list">
									<option>noreply</option>
								</datalist>
								<input type="text" id="black-user" list="black-list" name="user" placeholder="noreply" autocomplete="off" required="required" pattern="<?php echo $pattern; ?>"/>@
							</td>
							<td>
								<select id="black-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td></td>
							<td>
								<button type="submit" class="button-primary" name="type" value="email"><?php _ex( 'Add Blackhole', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			}

				if ( $edit !== 'responder' ) { ?>
			<form id="default-black" class="def-black" name="default-black" method="post" action="">
				<input type="hidden" name="action" value="default-black">
				<table class="form-table">
					<thead>
						<tr>
							<th></th>
							<th scope="col">
								<label for="default-black-domain"><?php _ex( 'For Domain', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>*@</td>
							<td>
								<select id="default-black-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td></td>
							<td>
								<button type="submit" class="button-primary"><?php _ex( 'Change Default to Blackhole', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			}

				if ( ! $edit ) { ?>
			<p class="new-black def-black"><?php _e( '&mdash; OR &mdash;' ); ?></p>
			<h2 class="new-acc"><big style="color: Green;">✉</big> <?php _ex( 'New Account &amp; Mailbox', 'Form Heading' ); ?></h2>
			<script>
				function validate() {
					let usr = document.forms['new-account']['user'].value;
					let dom = document.forms['new-account']['domain'].value;
//					alert( usr + '@' + dom );
					if ( [ '<?=\implode('\',\'',$used_emails)?>' ].includes( usr + '@' + dom ) ) {
						alert( '<?php _ex( 'Duplicate Email', 'Alert Message' ); ?>' );
						return false;
					} else {
						return true;
					}
				}
			</script>
			<form id="new-account" class="new-acc" method="post" action="" onsubmit="return validate();">
				<input type="hidden" name="action" value="new-account">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="new-email"><?php _ex(' Email Address', 'Form Field Label' ); ?>
							</th>
							<th></th>
							<th scope="col">
								<label for="password"><?php _ex( 'Account Password', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="new-email" name="user" autocomplete="on" required="required" pattern="<?php echo $pattern; ?>"/>@
							</td>
							<td>
								<select id="email-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td>
								<input type="password" id="password" name="password" minlength="8" autocomplete="new-password" placeholder="<?php _ex( 'Generate it', 'Password' ); ?>" title="<?php _ex( 'At least 8 characters, or empty to generate and send', 'Title text' ); ?>"/>
							</td>
							<td></td>
						</tr>
						<tr>
							<th colspan="2" scope="col" style="text-align: left;">
								<label for="name"><?php _ex( 'Send setup instructions to (name) (optional)', 'Form Field Label' ); ?>
							</th>
							<th scope="col">
								<label for="to"><?php _ex( 'To their current email address', 'Form Field Label' ); ?></label>
							</th>
							<th>
							</th>
						</tr>
						<tr>
							<td colspan="2" style="text-align: left;">
								<input type="text" id="name" name="name" placeholder="<?php echo \wp_get_current_user()->display_name; ?>"/>
							</td>
							<td style="padding: 0 1ch;">
								<input type="email" id="to" name="to" placeholder="<?php _ex( 'Do not send', 'Send to email input placeholder' ); ?>"/>
							</td>
							<td>
								<button type="submit" class="button-primary"><?php _ex( 'Create New Account', 'Submit button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<p<?php if ( ! $edit ) echo ' class="new-resp"'; ?><?php _e( '&mdash; OR &mdash;' ); ?></p>
<?php			}
				$type = 'responder';

				if ( ! $edit || $edit === 'responder' ) { ?>
			<h2<?php if ( ! $edit ) echo ' class="new-resp"'; ?>>↩️ <?php if ( $edit ) _ex( 'Edit Autoresponder', 'Form Heading' ); else _ex( 'New Autoresponder', 'Form Heading' ); ?></h2>
			<form id="<?php echo $edit ? 'edit' : 'new'; ?>-<?php echo $type; ?>" class="new-resp" method="post" action="">
				<input type="hidden" name="action" value="<?php echo $edit ? 'edit' : 'new'; ?>-<?php echo $type; ?>"/>
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="<?php echo $type; ?>-user"><?php _ex( 'For Email', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
							<th colspan="2" scope="col">
								<label for="<?php echo $type; ?>-from"><?php _ex( 'Responder Name', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="responder-user" name="user" autocomplete="on" required="required" value="<?php echo $responder->user; ?>" pattern="<?php echo $pattern; ?>"/>@
							</td>
							<td>
								<select id="<?php echo $type; ?>-domain" name="domain" required="required"<?=$readonly?>>
<?php
								foreach ( $mail_domains as $mail_domain ) { ?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								} ?>
								</select>
							</td>
							<td colspan="2">
								<input type="text" id="<?php echo $type; ?>-from" name="from" list="from-list" autocomplete="off" placeholder="<?php _ex( 'Name', 'Form Field Placeholder'); ?>" required="required" value="<?php echo $responder->from; ?>" pattern="^[a-zA-Z]+(?:\s+[a-zA-Z]+)*$">
								<datalist id="from-list">
								<?php foreach ( \get_users( [
									'number'  => \max( \count( $to_emails ), 20 ),
									'orderby' => 'display_name',
								] ) as $list_user ) {
									if ( \in_array( \explode( '@', $list_user->user_email )[1], $mail_domains, true ) ) { ?>
									<option><?php echo $list_user->display_name; ?></option>
									<?php }
								}
								foreach ( $dest_emails as $dest_email ) { ?>
									<option><?php echo \mb_ucwords( \str_replace( '.', ' ', \explode( '@', $dest_email )[0] ) ); ?>
								<?php } ?>
								</datalist>
							</td>
							<td></td>
						</tr>
						<tr class="thead">
							<th colspan="3" scope="col" style="text-align: left;">
								<label for="<?php echo $type; ?>-subject"><?php _ex( 'Subject', 'Form Field Label' ); ?></label>
							</th>
							<th scope="col" title="<?php _ex( 'Interval before repeated to same recipient.', 'Form Field Tooltip' ); ?>">
								<label for="<?php echo $type; ?>-interval"><?php _ex( 'Interval', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
						<tr>
							<td colspan="3" style="text-align: left;">
								<input id="<?php echo $type; ?>-subject" type="text" name="subject" value="<?php echo $responder->subject; ?>" style="width: 50ch;" placeholder="Re: %subject%" autocomplete="off" list="subject-list" required="required"/>
								<datalist id="subject-list">
									<option>Re: %subject%</option>
								</datalist>
							</td>
							<td>
								<input id="<?php echo $type; ?>-interval" type="number" name="interval" value="<?php echo $responder->interval; ?>" style="width: 8ch;" autocomplete="off" required="required" title="<?php _ex( 'Interval before repeated to same recipient.', 'Form Field Tooltip' ); ?>"/> <?php _ex( 'hours', 'Input Field Unit' ); ?>
							</td>
							<td></td>
						</tr>
						<tr class="thead">
							<th colspan="2" scope="col" style="text-align: left;">
								<label for="<?php echo $type; ?>-start"><?php _ex( 'Start time', 'Form Field Label' ); ?></label>
							</th>
							<th colspan="2" scope="col">
								<label for="<?php echo $type; ?>-stop"><?php _ex( 'End time', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
						<tr>
							<td colspan="2" style="text-align: left;">
								<input id="<?php echo $type; ?>-start-date" type="date" name="start-date" value="<?php echo $responder->start_date; ?>" style="width: 16ch;" autocomplete="off" required="required"/>
								<input type="time" name="start-time" value="<?php echo $responder->start_time; ?>" style="width: 12ch;"/>
							</td>
							<td colspan="2">
								<input id="<?php echo $type; ?>-start-date" type="date" name="stop-date" value="<?php echo $responder->stop_date; ?>" style="width: 15ch;" autocomplete="off" required="required"/>
								<input type="time" name="stop-time" value="<?php echo $responder->stop_time; ?>" style="width: 11ch;"/>
							</td>
							<td></td>
						</tr>
						<tr class="thead">
							<th colspan="4" scope="col" style="text-align: left;">
								<label for="<?php echo $type; ?>-body"><?php _ex( 'Email Message', 'Form Field Label' ); ?></label>
							</th>
							<th></th>
						</tr>
						<tr>
							<td colspan="4" style="text-align: left;">
								<textarea id="<?php echo $type; ?>-body" name="body" rows="6" required="required" style="width: calc(100% - 2ch);"><?php echo $responder->body; ?></textarea>
							</td>
							<td style="vertical-align: bottom;">
								<button type="submit" class="button-primary"><?php if ( $edit ) _ex( 'Save Autoresponder', 'Button' ); else _ex( 'Create New Autoresponder', 'Button' ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
<?php			} ?>
		</div>
<?php
				\add_action( 'admin_notices', static function(): void {
					$action = \sanitize_text_field( $_GET[ 'action' ] ?? '' );

					if ( $action ) { ?>
						<div class="notice notice-success is-dismissible">
							<p><?php echo \esc_attr( \ucfirst( $action ) ); ?> created!</p>
						</div>
<?php
					}
				} );
			}, 1 );
		} );
	}

	private   static function save_options(): void {

		if ( $_SERVER['REQUEST_METHOD'] === 'POST' && \sanitize_text_field( $_GET['page'] ?? '' ) === self::$page->NewEmail ) {
			$action  = \sanitize_key( $_POST['action' ] ?? '' );

			if ( $action === 'user-options' ) {

				if ( \wp_verify_nonce( $_POST['screenoptionnonce'], 'screen-options-nonce' ) ) {
					$user_id = \get_current_user_id();

					foreach ( self::$screen->get_options() as $option ) {
						$value = \boolval( $_POST[ $option['option'] ] ?? '' );

						if ( $value === \boolval( $option['default'] ?? '' ) ) {
							\delete_user_option( $user_id, $option['option'] );
						} else {
							\update_user_option( $user_id, $option['option'], $value );
						}
					}
				} else {
					\wp_die( _x( 'Expired link or invalid origin. Please go back, refresh and try again.', 'Die message' ) );
				}
			}
		}
	}

	private   static function save(): void {

		if ( $_SERVER['REQUEST_METHOD'] === 'POST' && \sanitize_text_field( $_GET['page'] ?? '' ) === self::$page->NewEmail ) {
			$action  = \sanitize_key( $_POST['action' ] ?? '' );
			$user    = \sanitize_text_field( $_POST['user'   ] ?? '' );
			$domain  = \sanitize_text_field( $_POST['domain' ] ?? '' );

			if ( $action && $domain && self::$domain_only && $domain !== self::$site_domain ) {
				\wp_die( \wpautop( \sprintf( _x( 'Invalid domain %1$s.', 'Die message' ), $domain ) ) );
			}
			$email   = $user && $domain ? \sanitize_email( $user . '@' . $domain ) : '';

			if ( false && self::$batch_mode ) {
//				self::batch_entry( $action, $email );
			} else {

				if ( $action === 'new-forward' ) {
					$forward = \sanitize_text_field( $_POST['forward'] ?? '' );
					self::notice( $action, UAPI::add_forwarder( $email, $forward ), $domain, $email );
				} elseif ( $action === 'default-forward' ) {
					$forward = \sanitize_email( $_POST['forward'] ?? '' );
					self::notice( $action, UAPI::set_default_email( $domain, $forward ), $domain, $forward );
				} elseif ( $action === 'new-failure' ) {
					$message = \sanitize_text_field( $_POST['msg'] ?? '' );
					self::notice( $action, UAPI::add_fail( $email, $message ), $domain, $email );
				} elseif ( $action === 'default-fail' ) {
					$message = \sanitize_text_field( $_POST['msg'] ?? '' );
					self::notice( $action, UAPI::set_default_fail( $domain, $message ), $domain );
				} elseif ( $action === 'new-black' ) {
					self::notice( $action, UAPI::add_blackhole( $email ), $domain, $email );
				} elseif ( $action === 'default-black' ) {
					self::notice( $action, UAPI::set_default_blackhole( $domain ), $domain );
				} elseif ( $action === 'new-account' ) {
					$current_user = \wp_get_current_user();
					$password = \sanitize_text_field( $_POST['password'] ?? '' );
					$name     = \mb_ucwords( \sanitize_text_field( $_POST['name'] ?? '' ) );
					$to       = \sanitize_email( $_POST['to'] ?? '' );

					if ( empty( $password ) ) {
						$password = \wp_generate_password( 12, false );
						$name = $name ?: $current_user->display_name;
						$to   = $to   ?: $current_user->user_email;
					}

					if ( $to ) {
						$result = UAPI::add_account( $email, $password );

						if ( $result->has_errors() ) {
							self::notice( $action, $result, $domain, $email );
						} else {
							$s_result = UAPI::send_settings( $email, $to );
							error_log( $s_result );
							$to = $name ? $name . ' <' . $to . '>' : $to;
							$domain = \explode( '@', $email )[1];
							$e_result = \wp_mail(
								$to,
								'[' . \get_bloginfo() . '] ' . _x( 'New Email Account created', 'Email subject' ),
								__( 'Email Account:' ) . ' ' . $email . \PHP_EOL .
								__( 'Email Account Password:' ) . ' ' . $password . \PHP_EOL . \PHP_EOL .
									__( 'Note 1: Not for login to the site!' ) . \PHP_EOL .
									\sprintf(
										_x( 'Note 2: You should also have received your client configuration settings for [%1$s].',
											'%1$s = domain' ),
										$domain
									) . \PHP_EOL .
									\sprintf(
										_x( 'Note 3: You should change this password from your webmail at %1$s', '%1$s = Webmail URL' ),
										self::$is_proisp ? 'https://webmail.proisp.no/' : 'https://mail.' . $domain . ':2096/',
									) . \PHP_EOL .
								self::guides(),
								[ 'From: ' . $current_user->display_name . ' <' . $current_user->user_email . '>' ]
							);
							error_log( $e_result );
							self::notice( $action, $result, $domain, $email );
						}
					} else {
						self::notice( $action, UAPI::add_account( $email, $password ), $domain, $email );
					}
				} elseif ( $action === 'new-responder' || $action === 'edit-responder' ) {
					$from      = \sanitize_text_field( $_POST['from' ] ?? '' );
					$subject   = \sanitize_text_field( $_POST['subject' ] ?? '' );
					$body      = \sanitize_textarea_field( $_POST['body' ] ?? '' );
					$body      = \str_replace( '%responder%', $from, $body );
					$start     = \mysql2date( 'U', $_POST['start-date'] . ' ' . $_POST['start-time'] ) - self::$tz_offset;
					$stop      = \mysql2date( 'U', $_POST['stop-date' ] . ' ' . $_POST['stop-time' ] ) - self::$tz_offset;
					$interval  = \intval( $_POST['interval'] );
					self::notice( $action, UAPI::add_responder( $email, $from, $subject, $body, $start, $stop, $interval ), $domain, $email );
				}
			}
			self::delete_transients();
		}
	}

	private static function batch_entry( string $action, string $email ): void {
		$action = \substr( $action, 4 );
		$bacth = self::$batch_entries;

		if ( $action === 'forward' ) {
			$forward = \sanitize_text_field( $_POST['forward'] ?? '' );
			$batch[] = $action . ': ' . $email . ' ' . $forward;
		} elseif ( $action === 'account' ) {
			$password = \sanitize_text_field( $_POST['password'] ?? '' );
			$name     = \mb_ucwords( \sanitize_text_field( $_POST['name'] ?? '' ) );
			$to       = \sanitize_email( $_POST['to'] ?? '' );

			if ( $password ) {
				$email = \str_replace( '@', ':' . $password . '@', $email );
			}

			if ( $to && $name ) {
				$to = $name . ' <' . $to . '>';
			}
			$batch[] = $action . ': ' . $email . ' ' . $to;
		}
		self::$batch_entries = $batch;
	}

	private static function notice( string $action, \WP_Error $result, string $domain = '', string $email = '' ): void {

		if ( $result->has_errors() ) {
			\add_action( 'admin_notices', static function() use ( $result ): void {
				\printf( '<div class="notice notice-error"><p>%1$s</p></div>', \esc_html( $result->get_error_message() ) );
			} );
		} else {
			\add_action( 'current_screen', static function() use ( $action, $domain, $email ): void {
				\wp_redirect( \add_query_arg( [ 'source' => self::$page->NewEmail, 'action' => $action, 'domain' => $domain, 'email' => $email ], \menu_page_url( self::$page->AccountsPage ) ) );
				exit;
			}, 999 );
		}
	}

	private static function guides() {
		$guides['en_US'] = [
			__( 'IMAP vs POP3',    'New account guide' ) => 'https://www.proisp.eu/faq/difference-between-imap-and-pop/',
			__( 'GMail',           'New account guide' ) => 'https://www.proisp.eu/faq/set-up-email-for-reading-sending-gmail/',
			__( 'Android',         'New account guide' ) => 'https://www.proisp.eu/guides/setup-email-account-android/',
			__( 'Windows',         'New account guide' ) => 'https://www.proisp.eu/guides/emailaccount-windows-10/',
			__( 'iPhone',          'New account guide' ) => 'https://www.proisp.eu/guides/setup-sent-folder-email-iphone-ipad/',
			__( 'Office 365',      'New account guide' ) => 'https://www.proisp.eu/guides/setup-email-account-outlook-office-365/',
			__( 'Android Outlook', 'New account guide' ) => 'https://www.proisp.eu/guides/setup-email-account-outlook-android/',
//			__( 'Android GMail',   'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epost-gmail-app-android/',
			__( 'Outlook 2016',    'New account guide' ) => 'https://www.proisp.eu/guides/setup-email-account-outlook-2013-2016/',
			__( 'Apple',           'New account guide' ) => 'https://www.proisp.eu/guides/create-email-account-apple-mail/',
			__( 'Thunderbird',     'New account guide' ) => 'https://www.proisp.eu/guides/setup-email-account-mozilla-thunderbird/',
			__( 'Webmail',         'New account guide' ) => 'https://www.proisp.eu/guides/using-webmail-cpanel-web-hosting/',
		];
		$guides['nb_NO'] = [
			__( 'IMAP vs POP3',    'New account guide' ) => 'https://www.proisp.no/oss/hva-er-forskjell-mellom-imap-og-pop/',
			__( 'GMail',           'New account guide' ) => 'https://www.proisp.no/oss/sette-opp-epost-for-lesing-og-sending-hos-gmail/',
			__( 'Android',         'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-android/',
			__( 'Windows',         'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-windows-8-mail/',
			__( 'iPhone',          'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-iphone-ipad/',
			__( 'iPhone GMail',    'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epost-gmail-app-iphone/',
			__( 'Office 365',      'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-outlook-office-365/',
			__( 'Android Outlook', 'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-outlook-android/',
			__( 'Android GMail',   'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epost-gmail-app-android/',
			__( 'Outlook 2016',    'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-outlook-2013-2016/',
			__( 'Apple',           'New account guide' ) => 'https://www.proisp.no/guider/opprettelse-av-epostkonto-apple-mail/',
			__( 'Thunderbird',     'New account guide' ) => 'https://www.proisp.no/guider/oppsett-epostkonto-mozilla-thunderbird/',
			__( 'Webmail',         'New account guide' ) => 'https://www.proisp.no/guider/bruk-webmail/',
		];
		$locale = \get_user_locale( \get_current_user_id() );
		$locale = \str_ends_with( $locale, '_NO' ) ? 'nb_NO' : 'en_US';
		$text = \PHP_EOL . __( 'Guides:' );

		foreach( $guides[ $locale ] as $label => $link ) {
			$text .= \PHP_EOL . $label . ': ' . $link;
		}
		return $text;
	}
}
