<?php
declare( strict_types=1 );
namespace WebFacing\cPanel\Email;
use WebFacing\cPanel\UAPI;

/**
 * Exit if accessed directly
 */
\class_exists( __NAMESPACE__ . '\Main' ) || exit;

abstract class ShortCode extends Main {

	public    static function load(): void {

		\add_shortcode( Main::pf . 'webmail', static function( /*string|array */$atts, ?string $content = null, ?string $tag = null ): string {
			Main::init();

			if ( UAPI::has_features( [ 'webmail' ] ) ) {

				\extract( \shortcode_atts( [
					'email'         => \is_user_logged_in() ? \wp_get_current_user()->user_email : '',
					'target'        => \apply_filters( Main::pf . 'shortcode_webmail_target',      '_blank', $atts ),
					'before_form'   => \apply_filters( Main::pf . 'shortcode_webmail_before_form', '',       $atts ),
					'after_form'    => \apply_filters( Main::pf . 'shortcode_webmail_after_form',  '',       $atts ),
					'form_class'    => \apply_filters( Main::pf . 'shortcode_webmail_form_class',   [ Main::$pf . 'webmail' ],    $atts ),
					'button_class'  => \apply_filters( Main::pf . 'shortcode_webmail_button_class', [ 'open-webmail', 'button' ], $atts ),
					'not_logged_in' => \apply_filters( Main::pf . 'shortcode_webmail_not_logged_in', _x( 'Not logged in',          'Shortcode fallback text' ), $atts ),
					'no_account'    => \apply_filters( Main::pf . 'shortcode_webmail_no_account',    _x( 'No cPanel® account yet', 'Shortcode fallback text' ), $atts ),
					], (array) $atts )
				);

				if ( \sanitize_email( $email ) ) {
					$content = \apply_filters( Main::pf . 'shortcode_webmail_button_label', $content ) ?: __( 'cPanel® Webmail', 'Button Label' );
					$user   = \explode( '@', $email )[0];
					$domain = \explode( '@', $email )[1] ?? '';

					if ( $domain === self::$main_domain && $user === self::$cpanel_user ) {
						$session = UAPI::create_main_webmail_session();
					} else {
						$session = UAPI::create_webmail_session( $email );
					}

					if ( ( $session->token ?? '' ) && ( $session->session ?? '' ) ) {
						$action = 'https://' . $session->hostname . ':2096' . $session->token . '/login';
						return $before_form . \strip_tags( '<form method="post" action="' . $action . '" target="' . $target . '" class="' . \implode( ' ', $form_class ) . '"><input type="hidden" name="session" value="' . $session->session . '"/><button type="submit" class="' . \implode( ' ', $button_class ) .'">' . $content . '</button></form>', [ 'form', 'input', 'button' ] ) . $after_form;
					} else {
						return \wp_kses_post( $no_account );
					}
				} else {
					return \wp_kses_post( $not_logged_in );
				}
			} else {
				return _x( 'No Webmail feature available in cPanel®', 'Shortcode fallback text' );
			}
		} );
	}
}